@@ -49,21 +49,26 @@ from lstchain.paths import parse_datacheck_dl1_filename, parse_dl1_filename, \
 
 def check_dl1(filenames, output_path, max_cores=4, create_pdf=False, batch=False):
     """
+    Check DL1 files
 
     Parameters
     ----------
-    batch: bool, run in batch mode
-    create_pdf: bool, create PDF file
-    filenames: string, Path, or a list of them, _sorted_ (by growing subrun
-    index). Name(s) of the input DL1 .h5 file(s)
-    output_path: directory where output will be written
-    max_cores: maximum number of processes that the function will spawn (each
+    batch: bool
+        run in batch mode
+    create_pdf: bool
+        create PDF file
+    filenames: str, Path, or a list of them, _sorted_ (by growing subrun
+    index). 
+        Name(s) of the input DL1 .h5 file(s)
+    output_path: 
+        directory where output will be written
+    max_cores: int
+        maximum number of processes that the function will spawn (each
     processing a different subrun)
 
     Returns
     -------
     None
-
     """
 
     logger = logging.getLogger(__name__)
@@ -185,17 +190,20 @@ def check_dl1(filenames, output_path, max_cores=4, create_pdf=False, batch=False
 
 def process_dl1_file(filename, bins, tel_id=1):
     """
+    Process DL1 file
 
     Parameters
     ----------
-    filename: string, or Path, input DL1 .h5 file to be checked
-    bins: DL1DataCheckHistogramBins container indicating binning of histograms
+    filename: str, or Path
+        input DL1 .h5 file to be checked
+    bins: DL1DataCheckHistogramBins container
+        indicating binning of histograms
     tel_id: int
         Telescope ID (default=1)
 
     Returns
     -------
-    dl1datacheck_pedestals, dl1datacheck_flatfield, dl1datacheck_cosmics
+    `dl1datacheck_pedestals`, `dl1datacheck_flatfield`, `dl1datacheck_cosmics`
     Containers of type DL1DataCheckContainer, with info on the three types of
     events: interleaved pedestals, interleaved flatfield events, and cosmics.
     If one or more of them is None, it means they have not been filled,
@@ -302,16 +310,20 @@ def process_dl1_file(filename, bins, tel_id=1):
 
 def plot_datacheck(datacheck_filename, out_path=None, batch=False, muons_dir=None, tel_id=1):
     """
+    Plot datacheck
 
     Parameters
     ----------
-    datacheck_filename: list of strings, or pathlib.Path, name(s) of .h5
-    files produced by the function check_dl1, starting from DL1 event files
-    If it is a list of file names, we expect each of the files to correspond to
-    one subrun of the same run.
-    out_path: optional; if not given, it will be the same of file filename
-    batch: bool, run in batch mode
-    muons_dir
+    datacheck_filename: list of strings, or pathlib.Path
+        name(s) of .h5 files produced by the function check_dl1, starting from
+        DL1 event files. If it is a list of file names, we expect each of the
+        files to correspond to one subrun of the same run.
+    out_path: str
+        optional; if not given, it will be the same of file filename
+    batch: bool
+        run in batch mode
+    muons_dir: str
+        directory for muon fits files
     tel_id: int
         Telescope ID (default=1)
 
@@ -1000,20 +1012,23 @@ def plot_datacheck(datacheck_filename, out_path=None, batch=False, muons_dir=Non
 
 def plot_trigger_types(dchecktables, trigger_name, axes):
     """
+    Plot trigger types
 
     Parameters
     ----------
     dchecktables: array of python tables created with DL1DataCheckContainer
-    containers (each row is one subrun). The plotted trigger type statistics
-    will be the global ones, adding up the numbers from all the tables and
-    all the rows in each table.
-    Inside the table the trigger type columns have shape (n,10,2). n is the
-    number of rows (one per subrun). 10 is the number of possible trigger
-    types (just fixed to a safely large value). The remaining 2 are the pairs
-    (trigger_id, number of entries with that id)
+    containers (each row is one subrun).
+        The plotted trigger type statistics will be the global ones, adding
+        up the numbers from all the tables and all the rows in each table.
+        Inside the table the trigger type columns have shape (n,10,2). n is the
+        number of rows (one per subrun). 10 is the number of possible trigger
+        types (just fixed to a safely large value). The remaining 2 are the pairs
+        (trigger_id, number of entries with that id)
 
-    trigger_name: name of the trigger type column in the tables
-    axes: where to place the plots
+    trigger_name:
+        name of the trigger type column in the tables
+    axes:
+        where to place the plots
 
     Returns
     -------
@@ -1046,25 +1061,32 @@ def plot_trigger_types(dchecktables, trigger_name, axes):
 
 def plot_mean_and_stddev(table, camgeom, columns, labels, pagesize, batch=False, norm='lin'):
     """
+    The subrun-wise mean and std dev values are used to calculate the
+    run-wise (i.e. for all processed subruns which appear in the table)
+    counterparts of the same, which are then plotted.
+
     Parameters
     ----------
-    batch: bool, run in batch mode
-    table:  python table containing pixel-wise information to be displayed
-    camgeom: camera geometry
-    columns: list of 2 strings, columns of 'table', first one is the mean and
+    batch: bool
+        run in batch mode
+    table:
+        python table containing pixel-wise information to be displayed
+    camgeom:
+        camera geometry
+    columns:
+        list of 2 strings, columns of 'table', first one is the mean and
     the second the std deviation to be plotted
-    labels: plot titles
-    pagesize: [width, height] in cm
-    norm:  lin or log, z-scale of camera displays
+    labels: str
+        plot titles
+    pagesize: float, float
+        [width, height] in cm
+    norm:  str
+        lin or log, z-scale of camera displays
 
     Returns
     -------
     None
 
-    The subrun-wise mean and std dev values are used to calculate the
-    run-wise (i.e. for all processed subruns which appear in the table)
-    counterparts of the same, which are then plotted.
-
     """
 
     logger = logging.getLogger(__name__)
@@ -1120,14 +1142,18 @@ def plot_mean_and_stddev(table, camgeom, columns, labels, pagesize, batch=False,
 
 def write_error_page(tablename, pagesize):
     """
+    Write error page
 
     Parameters
     ----------
-    tablename: name of the table which has no entries to be plotted
-    pagesize: [width, height] (cm)
+    tablename: str
+        name of the table which has no entries to be plotted
+    pagesize: float, float
+        [width, height] (cm)
     Returns
     -------
     None
+
     """
     _, axes = plt.subplots(nrows=1, ncols=1, figsize=pagesize)
     plt.text(0.5, 0.5, 'Sorry, no ' + tablename + ' to plot here!',
@@ -1138,18 +1164,21 @@ def write_error_page(tablename, pagesize):
 
 def merge_dl1datacheck_files(file_list):
     """
+    Merge DL1 Datacheck files
 
     Parameters
     ----------
-    file_list: list of strings, names of files of the kind produced by
-    function check_dl1
+    file_list:
+        list of strings, names of files of the kind produced by
+        function check_dl1
 
     Returns
     -------
-    merged_filename: name of the .h5 file which contains all the rows of the
-    files in the list (in the tables cosmics, pedestals and flatfield)
-    The camera geometry, histogram_binnings and used_trigger_tag are copied
-    just from the first file
+    merged_filename: str
+        name of the .h5 file which contains all the rows of the
+        files in the list (in the tables cosmics, pedestals and flatfield)
+        The camera geometry, histogram_binnings and used_trigger_tag are
+        copied just from the first file
 
     """
 
