@@ -1,4 +1,3 @@
-from .intensity_fitter import MuonIntensityFitter
 from .plot_muon import plot_muon_event
 from .muon_analysis import (
     analyze_muon_event,
@@ -9,7 +8,6 @@ from .muon_analysis import (
 )
 
 __all__ = [
-    'MuonIntensityFitter',
     'analyze_muon_event',
     'tag_pix_thr',
     'create_muon_table',
