@@ -14,6 +14,7 @@ from ctapipe.image.morphology import number_of_islands
 from numpy import nan
 
 from ..reco.disp import disp_parameters_event
+from ..reco.utils import get_event_pos_in_camera
 
 __all__ = [
     'DL1MonitoringEventIndexContainer',
@@ -92,6 +93,7 @@ class DL1ParametersContainer(Container):
     dragon_time = Field(None, 'Dragon time event trigger')
     ucts_time = Field(None, 'UCTS time event trigger')
     tib_time = Field(None, 'TIB time event trigger')
+    ucts_jump = Field(False, 'UCTS info jumped (corrected)')
 
     mc_energy = Field(None, 'Simulated Energy', unit=u.TeV)
     log_mc_energy = Field(None, 'log of simulated energy/TeV')
@@ -219,7 +221,7 @@ class DL1ParametersContainer(Container):
         self.tel_pos_z = tel_pos[2]
 
     def set_source_camera_position(self, event, telescope):
-        source_pos = utils.get_event_pos_in_camera(event, telescope)
+        source_pos = get_event_pos_in_camera(event, telescope)
         self.src_x = source_pos[0]
         self.src_y = source_pos[1]
 
@@ -275,9 +277,9 @@ class MetaData(Container):
     """
     Some metadata
     """
-    SOURCE_FILENAMES = Field([], "filename of the source file")
     LSTCHAIN_VERSION = Field(None, "version of lstchain")
     CTAPIPE_VERSION = Field(None, "version of ctapipe")
+    CTAPIPE_IO_LST_VERSION = Field(None, "version of ctapipe_io_lst")
     CONTACT = Field(None, "Person or institution responsible for this data product")
 
 
