@@ -37,13 +37,12 @@ def plot_features(data, true_hadroness=False):
     """Plot the distribution of different features that characterize
     events, such as hillas parameters or MC data.
 
-    Parameters:
-    -----------
+    Parameters
+    ----------
     data: pandas DataFrame
-
-true_hadroness:
-    True: True gammas and proton events are plotted (they are separated using true hadroness).
-    False: Gammas and protons are separated using reconstructed hadroness (hadro_rec)
+    true_hadroness:
+        True: True gammas and proton events are plotted (they are separated using true hadroness).
+        False: Gammas and protons are separated using reconstructed hadroness (hadro_rec).
     """
     hadro = "reco_type"
     if true_hadroness:
@@ -276,15 +275,14 @@ def read_resolutions(filename):
 def plot_disp(data, true_hadroness=False):
     """Plot the performance of reconstructed position
 
-    Parameters:
-    -----------
+    Parameters
+    ----------
     data: pandas DataFrame
-
     true_hadroness: boolean
-    True: True gammas and proton events are plotted (they are separated
-    using true hadroness).
-    False: Gammas and protons are separated using reconstructed
-    hadroness (hadro_rec)
+        True: True gammas and proton events are plotted (they are separated
+        using true hadroness).
+        False: Gammas and protons are separated using reconstructed
+        hadroness (hadro_rec)
     """
     hadro = "reco_type"
     if true_hadroness:
@@ -312,7 +310,7 @@ def plot_disp(data, true_hadroness=False):
 
     plt.plot(bins, y, 'r--', linewidth=2)
 
-    plt.xlabel(r'$\\frac{disp\_norm_{gammas}-disp_{rec}}{disp\_norm_{gammas}}$', fontsize=15)
+    plt.xlabel(r'$\frac{disp\_norm_{gammas}-disp_{rec}}{disp\_norm_{gammas}}$', fontsize=15)
 
     plt.figtext(0.15, 0.7, 'Mean: ' + str(round(mu, 4)), fontsize=12)
     plt.figtext(0.15, 0.65, 'Std: ' + str(round(sigma, 4)), fontsize=12)
@@ -492,30 +490,49 @@ def plot_models_features_importances(path_models, config_file=None, axes=None, *
         config = read_configuration_file(config_file)
 
     if axes is None:
-        fig, axes = plt.subplots(1, 3, figsize=(20, 5))
+        fig, axes = plt.subplots(2, 2, figsize=(20, 20))
     else:
-        fig = axes[0].get_figure()
+        fig = axes.ravel()[0].get_figure()
 
     fig.suptitle('Features importances')
 
     ### Regression models ###
-    reg_features_names = config['regression_features']
+    energy_reg_features_names = config['energy_regression_features']
+    disp_reg_features_names = config['disp_regression_features']
+    disp_clf_features_names = config['disp_classification_features']
 
     energy = joblib.load(os.path.join(path_models, "reg_energy.sav"))
-    disp = joblib.load(os.path.join(path_models, "reg_disp_vector.sav"))
 
-    plot_importances(disp, reg_features_names, ax=axes[0], **kwargs)
-    axes[0].set_title("disp")
+    if config['disp_method'] == 'disp_vector':
+        ax = axes[0, 0]
+        disp = joblib.load(os.path.join(path_models, "reg_disp_vector.sav"))
+        plot_importances(disp, disp_reg_features_names, ax=ax, **kwargs)
+        ax.set_title("disp vector")
+        ax = axes[0, 1]
+        ax.axis("off")
 
-    plot_importances(energy, reg_features_names, ax=axes[1], **kwargs)
-    axes[1].set_title("energy")
+    elif config['disp_method'] == 'disp_norm_sign':
+        disp_norm = joblib.load(os.path.join(path_models, "reg_disp_norm.sav"))
+        disp_sign = joblib.load(os.path.join(path_models, "cls_disp_sign.sav"))
+
+        ax = axes[0, 0]
+        plot_importances(disp_norm, disp_reg_features_names, ax=ax, **kwargs)
+        ax.set_title("disp norm")
+        ax = axes[0, 1]
+        plot_importances(disp_sign, disp_clf_features_names, ax=ax, **kwargs)
+        ax.set_title("disp sign")
+
+    ax = axes[1, 0]
+    plot_importances(energy, energy_reg_features_names, ax=ax, **kwargs)
+    ax.set_title("energy")
 
     ### Classification model ###
-    clf_features_names = config['classification_features']
+    clf_features_names = config['particle_classification_features']
     clf = joblib.load(os.path.join(path_models, "cls_gh.sav"))
 
-    plot_importances(clf, clf_features_names, ax=axes[2], **kwargs)
-    axes[2].set_title("classification")
+    ax = axes[1, 1]
+    plot_importances(clf, clf_features_names, ax=ax, **kwargs)
+    ax.set_title("classification")
 
     fig.tight_layout()
 
@@ -562,7 +579,7 @@ def plot_energy_resolution(dl2_data, ax=None, bias_correction=False, cta_req_nor
 
     Parameters
     ----------
-   dl2_data: `pandas.DataFrame`
+    dl2_data: `pandas.DataFrame`
         Reconstructed MC events at DL2+ level.
     ax: `matplotlib.pyplot.axes` or None
     bias_correction: `bool`
